@@ -31,6 +31,7 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 #include <nanoarrow/nanoarrow.h>
+
 #include "common/utils.h"
 
 namespace adbc_validation {
@@ -55,6 +56,11 @@ struct Releaser {
   }
 };
 
+template <>
+struct Releaser<struct ArrowBuffer> {
+  static void Release(struct ArrowBuffer* buffer) { ArrowBufferReset(buffer); }
+};
+
 template <>
 struct Releaser<struct ArrowArrayView> {
   static void Release(struct ArrowArrayView* value) {
@@ -200,7 +206,7 @@ struct StreamReader {
 
 /// \brief Read an AdbcGetInfoData struct with RAII safety
 struct GetObjectsReader {
-  explicit GetObjectsReader(struct ArrowArrayView* array_view) : array_view_(array_view) {
+  explicit GetObjectsReader(struct ArrowArrayView* array_view) {
     // TODO: this swallows any construction errors
     get_objects_data_ = AdbcGetObjectsDataInit(array_view);
   }
@@ -214,7 +220,6 @@ struct GetObjectsReader {
   }
 
  private:
-  struct ArrowArrayView* array_view_;
   struct AdbcGetObjectsData* get_objects_data_;
 };
 
@@ -239,8 +244,9 @@ int MakeArray(struct ArrowArray* parent, struct ArrowArray* array,
               const std::vector<std::optional<T>>& values) {
   for (const auto& v : values) {
     if (v.has_value()) {
-      if constexpr (std::is_same<T, int8_t>::value || std::is_same<T, int16_t>::value ||
-                    std::is_same<T, int32_t>::value || std::is_same<T, int64_t>::value) {
+      if constexpr (std::is_same<T, bool>::value || std::is_same<T, int8_t>::value ||
+                    std::is_same<T, int16_t>::value || std::is_same<T, int32_t>::value ||
+                    std::is_same<T, int64_t>::value) {
         if (int errno_res = ArrowArrayAppendInt(array, *v); errno_res != 0) {
           return errno_res;
         }
@@ -265,6 +271,18 @@ int MakeArray(struct ArrowArray* parent, struct ArrowArray* array,
         if (int errno_res = ArrowArrayAppendBytes(array, view); errno_res != 0) {
           return errno_res;
         }
+      } else if constexpr (std::is_same<T, std::vector<std::byte>>::value) {
+        static_assert(std::is_same_v<uint8_t, unsigned char>);
+        struct ArrowBufferView view;
+        view.data.as_uint8 = reinterpret_cast<const uint8_t*>(v->data());
+        view.size_bytes = v->size();
+        if (int errno_res = ArrowArrayAppendBytes(array, view); errno_res != 0) {
+          return errno_res;
+        }
+      } else if constexpr (std::is_same<T, ArrowInterval*>::value) {
+        if (int errno_res = ArrowArrayAppendInterval(array, *v); errno_res != 0) {
+          return errno_res;
+        }
       } else {
         static_assert(!sizeof(T), "Not yet implemented");
         return ENOTSUP;
@@ -348,6 +366,9 @@ void CompareArray(struct ArrowArrayView* array,
       } else if constexpr (std::is_same<T, float>::value) {
         ASSERT_NE(array->buffer_views[1].data.data, nullptr);
         ASSERT_EQ(*v, array->buffer_views[1].data.as_float[i]);
+      } else if constexpr (std::is_same<T, bool>::value) {
+        ASSERT_NE(array->buffer_views[1].data.data, nullptr);
+        ASSERT_EQ(*v, ArrowBitGet(array->buffer_views[1].data.as_uint8, i));
       } else if constexpr (std::is_same<T, int8_t>::value) {
         ASSERT_NE(array->buffer_views[1].data.data, nullptr);
         ASSERT_EQ(*v, array->buffer_views[1].data.as_int8[i]);
@@ -376,6 +397,21 @@ void CompareArray(struct ArrowArrayView* array,
         struct ArrowStringView view = ArrowArrayViewGetStringUnsafe(array, i);
         std::string str(view.data, view.size_bytes);
         ASSERT_EQ(*v, str);
+      } else if constexpr (std::is_same<T, std::vector<std::byte>>::value) {
+        struct ArrowBufferView view = ArrowArrayViewGetBytesUnsafe(array, i);
+        ASSERT_EQ(v->size(), view.size_bytes);
+        for (int64_t i = 0; i < view.size_bytes; i++) {
+          ASSERT_EQ((*v)[i], std::byte{view.data.as_uint8[i]});
+        }
+      } else if constexpr (std::is_same<T, ArrowInterval*>::value) {
+        ASSERT_NE(array->buffer_views[1].data.data, nullptr);
+        struct ArrowInterval interval;
+        ArrowIntervalInit(&interval, ArrowType::NANOARROW_TYPE_INTERVAL_MONTH_DAY_NANO);
+        ArrowArrayViewGetIntervalUnsafe(array, i, &interval);
+
+        ASSERT_EQ(interval.months, (*v)->months);
+        ASSERT_EQ(interval.days, (*v)->days);
+        ASSERT_EQ(interval.ns, (*v)->ns);
       } else {
         static_assert(!sizeof(T), "Not yet implemented");
       }
@@ -392,4 +428,7 @@ void CompareSchema(
     struct ArrowSchema* schema,
     const std::vector<std::tuple<std::optional<std::string>, ArrowType, bool>>& fields);
 
+/// \brief Helper method to get the vendor version of a driver
+std::string GetDriverVendorVersion(struct AdbcConnection* connection);
+
 }  // namespace adbc_validation
