@@ -22,17 +22,23 @@ import (
 	"database/sql"
 	"database/sql/driver"
 	"fmt"
+	"io"
 	"strconv"
 	"strings"
 	"time"
 
 	"github.com/apache/arrow-adbc/go/adbc"
 	"github.com/apache/arrow-adbc/go/adbc/driver/internal"
-	"github.com/apache/arrow/go/v13/arrow"
-	"github.com/apache/arrow/go/v13/arrow/array"
+	"github.com/apache/arrow/go/v14/arrow"
+	"github.com/apache/arrow/go/v14/arrow/array"
 	"github.com/snowflakedb/gosnowflake"
 )
 
+const (
+	defaultStatementQueueSize  = 200
+	defaultPrefetchConcurrency = 10
+)
+
 type snowflakeConn interface {
 	driver.Conn
 	driver.ConnBeginTx
@@ -45,11 +51,12 @@ type snowflakeConn interface {
 
 type cnxn struct {
 	cn    snowflakeConn
-	db    *database
+	db    *databaseImpl
 	ctor  gosnowflake.Connector
 	sqldb *sql.DB
 
 	activeTransaction bool
+	useHighPrecision  bool
 }
 
 // Metadata methods
@@ -95,18 +102,20 @@ type cnxn struct {
 // codes (the row will be omitted from the result).
 func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.RecordReader, error) {
 	const strValTypeID arrow.UnionTypeCode = 0
+	const intValTypeID arrow.UnionTypeCode = 2
 
 	if len(infoCodes) == 0 {
 		infoCodes = infoSupportedCodes
 	}
 
-	bldr := array.NewRecordBuilder(c.db.alloc, adbc.GetInfoSchema)
+	bldr := array.NewRecordBuilder(c.db.Alloc, adbc.GetInfoSchema)
 	defer bldr.Release()
 	bldr.Reserve(len(infoCodes))
 
 	infoNameBldr := bldr.Field(0).(*array.Uint32Builder)
 	infoValueBldr := bldr.Field(1).(*array.DenseUnionBuilder)
-	strInfoBldr := infoValueBldr.Child(0).(*array.StringBuilder)
+	strInfoBldr := infoValueBldr.Child(int(strValTypeID)).(*array.StringBuilder)
+	intInfoBldr := infoValueBldr.Child(int(intValTypeID)).(*array.Int64Builder)
 
 	for _, code := range infoCodes {
 		switch code {
@@ -122,6 +131,10 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 			infoNameBldr.Append(uint32(code))
 			infoValueBldr.Append(strValTypeID)
 			strInfoBldr.Append(infoDriverArrowVersion)
+		case adbc.InfoDriverADBCVersion:
+			infoNameBldr.Append(uint32(code))
+			infoValueBldr.Append(intValTypeID)
+			intInfoBldr.Append(adbc.AdbcVersion1_1_0)
 		case adbc.InfoVendorName:
 			infoNameBldr.Append(uint32(code))
 			infoValueBldr.Append(strValTypeID)
@@ -226,7 +239,7 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 // earlier).
 func (c *cnxn) GetObjects(ctx context.Context, depth adbc.ObjectDepth, catalog *string, dbSchema *string, tableName *string, columnName *string, tableType []string) (array.RecordReader, error) {
 	g := internal.GetObjects{Ctx: ctx, Depth: depth, Catalog: catalog, DbSchema: dbSchema, TableName: tableName, ColumnName: columnName, TableType: tableType}
-	if err := g.Init(c.db.alloc, c.getObjectsDbSchemas, c.getObjectsTables); err != nil {
+	if err := g.Init(c.db.Alloc, c.getObjectsDbSchemas, c.getObjectsTables); err != nil {
 		return nil, err
 	}
 	defer g.Release()
@@ -350,8 +363,9 @@ func (c *cnxn) getObjectsDbSchemas(ctx context.Context, depth adbc.ObjectDepth,
 
 var loc = time.Now().Location()
 
-func toField(name string, isnullable bool, dataType string, numPrec, numPrecRadix, numScale sql.NullInt16, isIdent bool, identGen, identInc, comment sql.NullString, ordinalPos int) (ret arrow.Field) {
+func toField(name string, isnullable bool, dataType string, numPrec, numPrecRadix, numScale sql.NullInt16, isIdent bool, identGen, identInc sql.NullString, charMaxLength, charOctetLength sql.NullInt32, datetimePrec sql.NullInt16, comment sql.NullString, ordinalPos int) (ret arrow.Field) {
 	ret.Name, ret.Nullable = name, isnullable
+
 	switch dataType {
 	case "NUMBER":
 		if !numScale.Valid || numScale.Int16 == 0 {
@@ -404,12 +418,98 @@ func toField(name string, isnullable bool, dataType string, numPrec, numPrecRadi
 	if comment.Valid {
 		md["COMMENT"] = comment.String
 	}
+
 	md["ORDINAL_POSITION"] = strconv.Itoa(ordinalPos)
+	md["XDBC_DATA_TYPE"] = strconv.Itoa(int(ret.Type.ID()))
+	md["XDBC_TYPE_NAME"] = dataType
+	md["XDBC_SQL_DATA_TYPE"] = strconv.Itoa(int(toXdbcDataType(ret.Type)))
+	md["XDBC_NULLABLE"] = strconv.FormatBool(isnullable)
+
+	if isnullable {
+		md["XDBC_IS_NULLABLE"] = "YES"
+	} else {
+		md["XDBC_IS_NULLABLE"] = "NO"
+	}
+
+	if numPrec.Valid {
+		md["XDBC_PRECISION"] = strconv.Itoa(int(numPrec.Int16))
+	}
+
+	if numScale.Valid {
+		md["XDBC_SCALE"] = strconv.Itoa(int(numScale.Int16))
+	}
+
+	if numPrec.Valid {
+		md["XDBC_PRECISION"] = strconv.Itoa(int(numPrec.Int16))
+	}
+
+	if numPrecRadix.Valid {
+		md["XDBC_NUM_PREC_RADIX"] = strconv.Itoa(int(numPrecRadix.Int16))
+	}
+
+	if charMaxLength.Valid {
+		md["CHARACTER_MAXIMUM_LENGTH"] = strconv.Itoa(int(charMaxLength.Int32))
+	}
+
+	if charOctetLength.Valid {
+		md["XDBC_CHAR_OCTET_LENGTH"] = strconv.Itoa(int(charOctetLength.Int32))
+	}
+
+	if datetimePrec.Valid {
+		md["XDBC_DATETIME_SUB"] = strconv.Itoa(int(datetimePrec.Int16))
+	}
 
 	ret.Metadata = arrow.MetadataFrom(md)
+
 	return
 }
 
+func toXdbcDataType(dt arrow.DataType) (xdbcType internal.XdbcDataType) {
+	xdbcType = internal.XdbcDataType_XDBC_UNKNOWN_TYPE
+	switch dt.ID() {
+	case arrow.EXTENSION:
+		return toXdbcDataType(dt.(arrow.ExtensionType).StorageType())
+	case arrow.DICTIONARY:
+		return toXdbcDataType(dt.(*arrow.DictionaryType).ValueType)
+	case arrow.RUN_END_ENCODED:
+		return toXdbcDataType(dt.(*arrow.RunEndEncodedType).Encoded())
+	case arrow.INT8, arrow.UINT8:
+		return internal.XdbcDataType_XDBC_TINYINT
+	case arrow.INT16, arrow.UINT16:
+		return internal.XdbcDataType_XDBC_SMALLINT
+	case arrow.INT32, arrow.UINT32:
+		return internal.XdbcDataType_XDBC_SMALLINT
+	case arrow.INT64, arrow.UINT64:
+		return internal.XdbcDataType_XDBC_BIGINT
+	case arrow.FLOAT32, arrow.FLOAT16, arrow.FLOAT64:
+		return internal.XdbcDataType_XDBC_FLOAT
+	case arrow.DECIMAL, arrow.DECIMAL256:
+		return internal.XdbcDataType_XDBC_DECIMAL
+	case arrow.STRING, arrow.LARGE_STRING:
+		return internal.XdbcDataType_XDBC_VARCHAR
+	case arrow.BINARY, arrow.LARGE_BINARY:
+		return internal.XdbcDataType_XDBC_BINARY
+	case arrow.FIXED_SIZE_BINARY:
+		return internal.XdbcDataType_XDBC_BINARY
+	case arrow.BOOL:
+		return internal.XdbcDataType_XDBC_BIT
+	case arrow.TIME32, arrow.TIME64:
+		return internal.XdbcDataType_XDBC_TIME
+	case arrow.DATE32, arrow.DATE64:
+		return internal.XdbcDataType_XDBC_DATE
+	case arrow.TIMESTAMP:
+		return internal.XdbcDataType_XDBC_TIMESTAMP
+	case arrow.DENSE_UNION, arrow.SPARSE_UNION:
+		return internal.XdbcDataType_XDBC_VARBINARY
+	case arrow.LIST, arrow.LARGE_LIST, arrow.FIXED_SIZE_LIST:
+		return internal.XdbcDataType_XDBC_VARBINARY
+	case arrow.STRUCT, arrow.MAP:
+		return internal.XdbcDataType_XDBC_VARBINARY
+	default:
+		return internal.XdbcDataType_XDBC_UNKNOWN_TYPE
+	}
+}
+
 func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, catalog *string, dbSchema *string, tableName *string, columnName *string, tableType []string) (result internal.SchemaToTableInfo, err error) {
 	if depth == adbc.ObjectDepthCatalogs || depth == adbc.ObjectDepthDBSchemas {
 		return
@@ -472,7 +572,8 @@ func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, cat
 				table_catalog, table_schema, table_name, column_name,
 				ordinal_position, is_nullable::boolean, data_type, numeric_precision,
 				numeric_precision_radix, numeric_scale, is_identity::boolean,
-				identity_generation, identity_increment, comment
+				identity_generation, identity_increment,
+				character_maximum_length, character_octet_length, datetime_precision, comment
 		FROM ' || rec.database_name || '.INFORMATION_SCHEMA.COLUMNS';
 
 		  counter := counter + 1;
@@ -540,11 +641,12 @@ func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, cat
 		defer rows.Close()
 
 		var (
-			colName, dataType                           string
-			identGen, identIncrement, comment           sql.NullString
-			ordinalPos                                  int
-			numericPrec, numericPrecRadix, numericScale sql.NullInt16
-			isNullable, isIdent                         bool
+			colName, dataType                                         string
+			identGen, identIncrement, comment                         sql.NullString
+			ordinalPos                                                int
+			numericPrec, numericPrecRadix, numericScale, datetimePrec sql.NullInt16
+			isNullable, isIdent                                       bool
+			charMaxLength, charOctetLength                            sql.NullInt32
 
 			prevKey      internal.CatalogAndSchema
 			curTableInfo *internal.TableInfo
@@ -556,7 +658,7 @@ func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, cat
 			err = rows.Scan(&tblCat, &tblSchema, &tblName, &colName,
 				&ordinalPos, &isNullable, &dataType, &numericPrec,
 				&numericPrecRadix, &numericScale, &isIdent, &identGen,
-				&identIncrement, &comment)
+				&identIncrement, &charMaxLength, &charOctetLength, &datetimePrec, &comment)
 			if err != nil {
 				err = errToAdbcErr(adbc.StatusIO, err)
 				return
@@ -579,7 +681,7 @@ func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, cat
 			}
 
 			prevKey = key
-			fieldList = append(fieldList, toField(colName, isNullable, dataType, numericPrec, numericPrecRadix, numericScale, isIdent, identGen, identIncrement, comment, ordinalPos))
+			fieldList = append(fieldList, toField(colName, isNullable, dataType, numericPrec, numericPrecRadix, numericScale, isIdent, identGen, identIncrement, charMaxLength, charOctetLength, datetimePrec, comment, ordinalPos))
 		}
 
 		if len(fieldList) > 0 && curTableInfo != nil {
@@ -674,6 +776,85 @@ func descToField(name, typ, isnull, primary string, comment sql.NullString) (fie
 	return
 }
 
+func (c *cnxn) GetOption(key string) (string, error) {
+	switch key {
+	case adbc.OptionKeyAutoCommit:
+		if c.activeTransaction {
+			// No autocommit
+			return adbc.OptionValueDisabled, nil
+		} else {
+			// Autocommit
+			return adbc.OptionValueEnabled, nil
+		}
+	case adbc.OptionKeyCurrentCatalog:
+		return c.getStringQuery("SELECT CURRENT_DATABASE()")
+	case adbc.OptionKeyCurrentDbSchema:
+		return c.getStringQuery("SELECT CURRENT_SCHEMA()")
+	}
+
+	return "", adbc.Error{
+		Msg:  "[Snowflake] unknown connection option",
+		Code: adbc.StatusNotFound,
+	}
+}
+
+func (c *cnxn) getStringQuery(query string) (string, error) {
+	result, err := c.cn.QueryContext(context.Background(), query, nil)
+	if err != nil {
+		return "", errToAdbcErr(adbc.StatusInternal, err)
+	}
+	defer result.Close()
+
+	if len(result.Columns()) != 1 {
+		return "", adbc.Error{
+			Msg:  fmt.Sprintf("[Snowflake] Internal query returned wrong number of columns: %s", result.Columns()),
+			Code: adbc.StatusInternal,
+		}
+	}
+
+	dest := make([]driver.Value, 1)
+	err = result.Next(dest)
+	if err == io.EOF {
+		return "", adbc.Error{
+			Msg:  "[Snowflake] Internal query returned no rows",
+			Code: adbc.StatusInternal,
+		}
+	} else if err != nil {
+		return "", errToAdbcErr(adbc.StatusInternal, err)
+	}
+
+	value, ok := dest[0].(string)
+	if !ok {
+		return "", adbc.Error{
+			Msg:  fmt.Sprintf("[Snowflake] Internal query returned wrong type of value: %s", dest[0]),
+			Code: adbc.StatusInternal,
+		}
+	}
+
+	return value, nil
+}
+
+func (c *cnxn) GetOptionBytes(key string) ([]byte, error) {
+	return nil, adbc.Error{
+		Msg:  "[Snowflake] unknown connection option",
+		Code: adbc.StatusNotFound,
+	}
+}
+
+func (c *cnxn) GetOptionInt(key string) (int64, error) {
+	return 0, adbc.Error{
+		Msg:  "[Snowflake] unknown connection option",
+		Code: adbc.StatusNotFound,
+	}
+}
+
+func (c *cnxn) GetOptionDouble(key string) (float64, error) {
+	return 0.0, adbc.Error{
+		Msg:  "[Snowflake] unknown connection option",
+		Code: adbc.StatusNotFound,
+	}
+}
+
 func (c *cnxn) GetTableSchema(ctx context.Context, catalog *string, dbSchema *string, tableName string) (*arrow.Schema, error) {
 	tblParts := make([]string, 0, 3)
 	if catalog != nil {
@@ -723,7 +904,7 @@ func (c *cnxn) GetTableSchema(ctx context.Context, catalog *string, dbSchema *st
 //	----------------|--------------
 //	table_type			| utf8 not null
 func (c *cnxn) GetTableTypes(_ context.Context) (array.RecordReader, error) {
-	bldr := array.NewRecordBuilder(c.db.alloc, adbc.TableTypesSchema)
+	bldr := array.NewRecordBuilder(c.db.Alloc, adbc.TableTypesSchema)
 	defer bldr.Release()
 
 	bldr.Field(0).(*array.StringBuilder).AppendValues([]string{"BASE TABLE", "TEMPORARY TABLE", "VIEW"}, nil)
@@ -777,8 +958,11 @@ func (c *cnxn) Rollback(_ context.Context) error {
 // NewStatement initializes a new statement object tied to this connection
 func (c *cnxn) NewStatement() (adbc.Statement, error) {
 	return &statement{
-		alloc: c.db.alloc,
-		cnxn:  c,
+		alloc:               c.db.Alloc,
+		cnxn:                c,
+		queueSize:           defaultStatementQueueSize,
+		prefetchConcurrency: defaultPrefetchConcurrency,
+		useHighPrecision:    c.useHighPrecision,
 	}, nil
 }
 
@@ -840,6 +1024,28 @@ func (c *cnxn) SetOption(key, value string) error {
 				Code: adbc.StatusInvalidArgument,
 			}
 		}
+	case adbc.OptionKeyCurrentCatalog:
+		_, err := c.cn.ExecContext(context.Background(), "USE DATABASE ?", []driver.NamedValue{{Value: value}})
+		return err
+	case adbc.OptionKeyCurrentDbSchema:
+		_, err := c.cn.ExecContext(context.Background(), "USE SCHEMA ?", []driver.NamedValue{{Value: value}})
+		return err
+	case OptionUseHighPrecision:
+		// statements will inherit the value of the OptionUseHighPrecision
+		// from the connection, but the option can be overridden at the
+		// statement level if SetOption is called on the statement.
+		switch value {
+		case adbc.OptionValueEnabled:
+			c.useHighPrecision = true
+		case adbc.OptionValueDisabled:
+			c.useHighPrecision = false
+		default:
+			return adbc.Error{
+				Msg:  "[Snowflake] invalid value for option " + key + ": " + value,
+				Code: adbc.StatusInvalidArgument,
+			}
+		}
+		return nil
 	default:
 		return adbc.Error{
 			Msg:  "[Snowflake] unknown connection option " + key + ": " + value,
@@ -847,3 +1053,24 @@ func (c *cnxn) SetOption(key, value string) error {
 		}
 	}
 }
+
+func (c *cnxn) SetOptionBytes(key string, value []byte) error {
+	return adbc.Error{
+		Msg:  "[Snowflake] unknown connection option",
+		Code: adbc.StatusNotImplemented,
+	}
+}
+
+func (c *cnxn) SetOptionInt(key string, value int64) error {
+	return adbc.Error{
+		Msg:  "[Snowflake] unknown connection option",
+		Code: adbc.StatusNotImplemented,
+	}
+}
+
+func (c *cnxn) SetOptionDouble(key string, value float64) error {
+	return adbc.Error{
+		Msg:  "[Snowflake] unknown connection option",
+		Code: adbc.StatusNotImplemented,
+	}
+}
