@@ -28,9 +28,9 @@ import (
 
 	"github.com/apache/arrow-adbc/go/adbc"
 	"github.com/apache/arrow-adbc/go/adbc/utils"
-	"github.com/apache/arrow/go/v13/arrow"
-	"github.com/apache/arrow/go/v13/arrow/array"
-	"github.com/apache/arrow/go/v13/arrow/memory"
+	"github.com/apache/arrow/go/v14/arrow"
+	"github.com/apache/arrow/go/v14/arrow/array"
+	"github.com/apache/arrow/go/v14/arrow/memory"
 	"github.com/stretchr/testify/suite"
 )
 
@@ -44,10 +44,20 @@ type DriverQuirks interface {
 	DatabaseOptions() map[string]string
 	// Return the SQL to reference the bind parameter for a given index
 	BindParameter(index int) string
+	// Whether the driver supports bulk ingest
+	SupportsBulkIngest(mode string) bool
 	// Whether two statements can be used at the same time on a single connection
 	SupportsConcurrentStatements() bool
+	// Whether current catalog/schema are supported
+	SupportsCurrentCatalogSchema() bool
+	// Whether GetSetOptions is supported
+	SupportsGetSetOptions() bool
+	// Whether AdbcStatementExecuteSchema should work
+	SupportsExecuteSchema() bool
 	// Whether AdbcStatementExecutePartitions should work
 	SupportsPartitionedData() bool
+	// Whether statistics are supported
+	SupportsStatistics() bool
 	// Whether transactions are supported (Commit/Rollback on connection)
 	SupportsTransactions() bool
 	// Whether retrieving the schema of prepared statement params is supported
@@ -60,11 +70,10 @@ type DriverQuirks interface {
 	CreateSampleTable(tableName string, r arrow.Record) error
 	// Field Metadata for Sample Table for comparison
 	SampleTableSchemaMetadata(tblName string, dt arrow.DataType) arrow.Metadata
-	// Whether the driver supports bulk ingest
-	SupportsBulkIngest() bool
 	// have the driver drop a table with the correct SQL syntax
 	DropTable(adbc.Connection, string) error
 
+	Catalog() string
 	DBSchema() string
 
 	Alloc() memory.Allocator
@@ -115,6 +124,30 @@ func (c *ConnectionTests) TearDownTest() {
 	c.DB = nil
 }
 
+func (c *ConnectionTests) TestGetSetOptions() {
+	cnxn, err := c.DB.Open(context.Background())
+	c.NoError(err)
+	c.NotNil(cnxn)
+
+	stmt, err := cnxn.NewStatement()
+	c.NoError(err)
+	c.NotNil(stmt)
+
+	expected := c.Quirks.SupportsGetSetOptions()
+
+	_, ok := c.DB.(adbc.GetSetOptions)
+	c.Equal(expected, ok)
+
+	_, ok = cnxn.(adbc.GetSetOptions)
+	c.Equal(expected, ok)
+
+	_, ok = stmt.(adbc.GetSetOptions)
+	c.Equal(expected, ok)
+
+	c.NoError(stmt.Close())
+	c.NoError(cnxn.Close())
+}
+
 func (c *ConnectionTests) TestNewConn() {
 	cnxn, err := c.DB.Open(context.Background())
 	c.NoError(err)
@@ -152,6 +185,12 @@ func (c *ConnectionTests) TestAutocommitDefault() {
 	cnxn, _ := c.DB.Open(ctx)
 	defer cnxn.Close()
 
+	if getset, ok := cnxn.(adbc.GetSetOptions); ok {
+		value, err := getset.GetOption(adbc.OptionKeyAutoCommit)
+		c.NoError(err)
+		c.Equal(adbc.OptionValueEnabled, value)
+	}
+
 	expectedCode := adbc.StatusInvalidState
 	var adbcError adbc.Error
 	err := cnxn.Commit(ctx)
@@ -188,8 +227,60 @@ func (c *ConnectionTests) TestAutocommitToggle() {
 	c.NoError(cnxnopt.SetOption(adbc.OptionKeyAutoCommit, adbc.OptionValueEnabled))
 	c.NoError(cnxnopt.SetOption(adbc.OptionKeyAutoCommit, adbc.OptionValueDisabled))
 
+	if getset, ok := cnxn.(adbc.GetSetOptions); ok {
+		value, err := getset.GetOption(adbc.OptionKeyAutoCommit)
+		c.NoError(err)
+		c.Equal(adbc.OptionValueDisabled, value)
+	}
+
 	// it is ok to disable autocommit when it isn't enabled
 	c.NoError(cnxnopt.SetOption(adbc.OptionKeyAutoCommit, adbc.OptionValueDisabled))
+
+	if getset, ok := cnxn.(adbc.GetSetOptions); ok {
+		value, err := getset.GetOption(adbc.OptionKeyAutoCommit)
+		c.NoError(err)
+		c.Equal(adbc.OptionValueDisabled, value)
+	}
+}
+
+func (c *ConnectionTests) TestMetadataCurrentCatalog() {
+	ctx := context.Background()
+	cnxn, _ := c.DB.Open(ctx)
+	defer cnxn.Close()
+	getset, ok := cnxn.(adbc.GetSetOptions)
+
+	if !c.Quirks.SupportsGetSetOptions() {
+		c.False(ok)
+		return
+	}
+	c.True(ok)
+	value, err := getset.GetOption(adbc.OptionKeyCurrentCatalog)
+	if c.Quirks.SupportsCurrentCatalogSchema() {
+		c.NoError(err)
+		c.Equal(c.Quirks.Catalog(), value)
+	} else {
+		c.Error(err)
+	}
+}
+
+func (c *ConnectionTests) TestMetadataCurrentDbSchema() {
+	ctx := context.Background()
+	cnxn, _ := c.DB.Open(ctx)
+	defer cnxn.Close()
+	getset, ok := cnxn.(adbc.GetSetOptions)
+
+	if !c.Quirks.SupportsGetSetOptions() {
+		c.False(ok)
+		return
+	}
+	c.True(ok)
+	value, err := getset.GetOption(adbc.OptionKeyCurrentDbSchema)
+	if c.Quirks.SupportsCurrentCatalogSchema() {
+		c.NoError(err)
+		c.Equal(c.Quirks.DBSchema(), value)
+	} else {
+		c.Error(err)
+	}
 }
 
 func (c *ConnectionTests) TestMetadataGetInfo() {
@@ -201,6 +292,7 @@ func (c *ConnectionTests) TestMetadataGetInfo() {
 		adbc.InfoDriverName,
 		adbc.InfoDriverVersion,
 		adbc.InfoDriverArrowVersion,
+		adbc.InfoDriverADBCVersion,
 		adbc.InfoVendorName,
 		adbc.InfoVendorVersion,
 		adbc.InfoVendorArrowVersion,
@@ -219,19 +311,55 @@ func (c *ConnectionTests) TestMetadataGetInfo() {
 		valUnion := rec.Column(1).(*array.DenseUnion)
 		for i := 0; i < int(rec.NumRows()); i++ {
 			code := codeCol.Value(i)
-
 			child := valUnion.Field(valUnion.ChildID(i))
-			if child.IsNull(i) {
+			offset := int(valUnion.ValueOffset(i))
+			valUnion.GetOneForMarshal(i)
+			if child.IsNull(offset) {
 				exp := c.Quirks.GetMetadata(adbc.InfoCode(code))
 				c.Nilf(exp, "got nil for info %s, expected: %s", adbc.InfoCode(code), exp)
 			} else {
-				// currently we only define utf8 values for metadata
-				c.Equal(c.Quirks.GetMetadata(adbc.InfoCode(code)), child.(*array.String).Value(i), adbc.InfoCode(code).String())
+				expected := c.Quirks.GetMetadata(adbc.InfoCode(code))
+				var actual interface{}
+
+				switch valUnion.ChildID(i) {
+				case 0:
+					// String
+					actual = child.(*array.String).Value(offset)
+				case 2:
+					// int64
+					actual = child.(*array.Int64).Value(offset)
+				default:
+					c.FailNow("Unknown union type code", valUnion.ChildID(i))
+				}
+
+				c.Equal(expected, actual, adbc.InfoCode(code).String())
 			}
 		}
 	}
 }
 
+func (c *ConnectionTests) TestMetadataGetStatistics() {
+	ctx := context.Background()
+	cnxn, _ := c.DB.Open(ctx)
+	defer cnxn.Close()
+
+	if c.Quirks.SupportsStatistics() {
+		stats, ok := cnxn.(adbc.ConnectionGetStatistics)
+		c.True(ok)
+		reader, err := stats.GetStatistics(ctx, nil, nil, nil, true)
+		c.NoError(err)
+		defer reader.Release()
+	} else {
+		stats, ok := cnxn.(adbc.ConnectionGetStatistics)
+		if ok {
+			_, err := stats.GetStatistics(ctx, nil, nil, nil, true)
+			var adbcErr adbc.Error
+			c.ErrorAs(err, &adbcErr)
+			c.Equal(adbc.StatusNotImplemented, adbcErr.Code)
+		}
+	}
+}
+
 func (c *ConnectionTests) TestMetadataGetTableSchema() {
 	rec, _, err := array.RecordFromJSON(c.Quirks.Alloc(), arrow.NewSchema(
 		[]arrow.Field{
@@ -407,6 +535,49 @@ func (s *StatementTests) TestNewStatement() {
 	s.Equal(adbc.StatusInvalidState, adbcError.Code)
 }
 
+func (s *StatementTests) TestSqlExecuteSchema() {
+	if !s.Quirks.SupportsExecuteSchema() {
+		s.T().SkipNow()
+	}
+
+	stmt, err := s.Cnxn.NewStatement()
+	s.Require().NoError(err)
+	defer stmt.Close()
+
+	es, ok := stmt.(adbc.StatementExecuteSchema)
+	s.Require().True(ok, "%#v does not support ExecuteSchema", es)
+
+	s.Run("no query", func() {
+		var adbcErr adbc.Error
+
+		schema, err := es.ExecuteSchema(s.ctx)
+		s.ErrorAs(err, &adbcErr)
+		s.Equal(adbc.StatusInvalidState, adbcErr.Code)
+		s.Nil(schema)
+	})
+
+	s.Run("query", func() {
+		s.NoError(stmt.SetSqlQuery("SELECT 1, 'string'"))
+
+		schema, err := es.ExecuteSchema(s.ctx)
+		s.NoError(err)
+		s.Equal(2, len(schema.Fields()))
+		s.True(schema.Field(0).Type.ID() == arrow.INT32 || schema.Field(0).Type.ID() == arrow.INT64)
+		s.Equal(arrow.STRING, schema.Field(1).Type.ID())
+	})
+
+	s.Run("prepared", func() {
+		s.NoError(stmt.SetSqlQuery("SELECT 1, 'string'"))
+		s.NoError(stmt.Prepare(s.ctx))
+
+		schema, err := es.ExecuteSchema(s.ctx)
+		s.NoError(err)
+		s.Equal(2, len(schema.Fields()))
+		s.True(schema.Field(0).Type.ID() == arrow.INT32 || schema.Field(0).Type.ID() == arrow.INT64)
+		s.Equal(arrow.STRING, schema.Field(1).Type.ID())
+	})
+}
+
 func (s *StatementTests) TestSqlPartitionedInts() {
 	stmt, err := s.Cnxn.NewStatement()
 	s.Require().NoError(err)
@@ -596,7 +767,7 @@ func (s *StatementTests) TestSqlPrepareErrorParamCountMismatch() {
 }
 
 func (s *StatementTests) TestSqlIngestInts() {
-	if !s.Quirks.SupportsBulkIngest() {
+	if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeCreate) {
 		s.T().SkipNow()
 	}
 
@@ -647,7 +818,7 @@ func (s *StatementTests) TestSqlIngestInts() {
 }
 
 func (s *StatementTests) TestSqlIngestAppend() {
-	if !s.Quirks.SupportsBulkIngest() {
+	if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeAppend) {
 		s.T().SkipNow()
 	}
 
@@ -683,6 +854,10 @@ func (s *StatementTests) TestSqlIngestAppend() {
 	defer batch2.Release()
 
 	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestTargetTable, "bulk_ingest"))
+
+	if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeAppend) {
+		s.T().SkipNow()
+	}
 	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestMode, adbc.OptionValueIngestModeAppend))
 	s.Require().NoError(stmt.Bind(s.ctx, batch2))
 
@@ -716,11 +891,151 @@ func (s *StatementTests) TestSqlIngestAppend() {
 	s.Require().NoError(rdr.Err())
 }
 
-func (s *StatementTests) TestSqlIngestErrors() {
-	if !s.Quirks.SupportsBulkIngest() {
+func (s *StatementTests) TestSqlIngestReplace() {
+	if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeReplace) {
 		s.T().SkipNow()
 	}
 
+	s.Require().NoError(s.Quirks.DropTable(s.Cnxn, "bulk_ingest"))
+
+	schema := arrow.NewSchema([]arrow.Field{{
+		Name: "int64s", Type: arrow.PrimitiveTypes.Int64, Nullable: true}}, nil)
+
+	batchbldr := array.NewRecordBuilder(s.Quirks.Alloc(), schema)
+	defer batchbldr.Release()
+	bldr := batchbldr.Field(0).(*array.Int64Builder)
+	bldr.AppendValues([]int64{42}, []bool{true})
+	batch := batchbldr.NewRecord()
+	defer batch.Release()
+
+	// ingest and create table
+	stmt, err := s.Cnxn.NewStatement()
+	s.Require().NoError(err)
+	defer stmt.Close()
+
+	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestTargetTable, "bulk_ingest"))
+	s.Require().NoError(stmt.Bind(s.ctx, batch))
+
+	affected, err := stmt.ExecuteUpdate(s.ctx)
+	s.Require().NoError(err)
+	if affected != -1 && affected != 1 {
+		s.FailNowf("invalid number of affected rows", "should be -1 or 1, got: %d", affected)
+	}
+
+	// now replace
+	schema = arrow.NewSchema([]arrow.Field{{
+		Name: "newintcol", Type: arrow.PrimitiveTypes.Int64, Nullable: true}}, nil)
+	batchbldr2 := array.NewRecordBuilder(s.Quirks.Alloc(), schema)
+	defer batchbldr2.Release()
+	bldr2 := batchbldr2.Field(0).(*array.Int64Builder)
+	bldr2.AppendValues([]int64{42}, []bool{true})
+	batch2 := batchbldr2.NewRecord()
+	defer batch2.Release()
+
+	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestTargetTable, "bulk_ingest"))
+	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestMode, adbc.OptionValueIngestModeReplace))
+	s.Require().NoError(stmt.Bind(s.ctx, batch2))
+
+	affected, err = stmt.ExecuteUpdate(s.ctx)
+	s.Require().NoError(err)
+	if affected != -1 && affected != 1 {
+		s.FailNowf("invalid number of affected rows", "should be -1 or 1, got: %d", affected)
+	}
+
+	s.Require().NoError(stmt.SetSqlQuery(`SELECT * FROM bulk_ingest`))
+	rdr, rows, err := stmt.ExecuteQuery(s.ctx)
+	s.Require().NoError(err)
+	if rows != -1 && rows != 1 {
+		s.FailNowf("invalid number of returned rows", "should be -1 or 1, got: %d", rows)
+	}
+	defer rdr.Release()
+
+	s.Truef(schema.Equal(utils.RemoveSchemaMetadata(rdr.Schema())), "expected: %s\n got: %s", schema, rdr.Schema())
+	s.Require().True(rdr.Next())
+	rec := rdr.Record()
+	s.EqualValues(1, rec.NumRows())
+	s.EqualValues(1, rec.NumCols())
+	col, ok := rec.Column(0).(*array.Int64)
+	s.True(ok)
+	s.Equal(int64(42), col.Value(0))
+
+	s.Require().False(rdr.Next())
+	s.Require().NoError(rdr.Err())
+}
+
+func (s *StatementTests) TestSqlIngestCreateAppend() {
+	if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeCreateAppend) {
+		s.T().SkipNow()
+	}
+
+	s.Require().NoError(s.Quirks.DropTable(s.Cnxn, "bulk_ingest"))
+
+	schema := arrow.NewSchema([]arrow.Field{{
+		Name: "int64s", Type: arrow.PrimitiveTypes.Int64, Nullable: true}}, nil)
+
+	batchbldr := array.NewRecordBuilder(s.Quirks.Alloc(), schema)
+	defer batchbldr.Release()
+	bldr := batchbldr.Field(0).(*array.Int64Builder)
+	bldr.AppendValues([]int64{42}, []bool{true})
+	batch := batchbldr.NewRecord()
+	defer batch.Release()
+
+	// ingest and create table
+	stmt, err := s.Cnxn.NewStatement()
+	s.Require().NoError(err)
+	defer stmt.Close()
+
+	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestTargetTable, "bulk_ingest"))
+	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestMode, adbc.OptionValueIngestModeCreateAppend))
+	s.Require().NoError(stmt.Bind(s.ctx, batch))
+
+	affected, err := stmt.ExecuteUpdate(s.ctx)
+	s.Require().NoError(err)
+	if affected != -1 && affected != 1 {
+		s.FailNowf("invalid number of affected rows", "should be -1 or 1, got: %d", affected)
+	}
+
+	// append
+	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestTargetTable, "bulk_ingest"))
+	s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestMode, adbc.OptionValueIngestModeCreateAppend))
+	s.Require().NoError(stmt.Bind(s.ctx, batch))
+
+	affected, err = stmt.ExecuteUpdate(s.ctx)
+	s.Require().NoError(err)
+	if affected != -1 && affected != 1 {
+		s.FailNowf("invalid number of affected rows", "should be -1 or 1, got: %d", affected)
+	}
+
+	// validate
+	s.Require().NoError(stmt.SetSqlQuery(`SELECT * FROM bulk_ingest`))
+	rdr, rows, err := stmt.ExecuteQuery(s.ctx)
+	s.Require().NoError(err)
+	if rows != -1 && rows != 2 {
+		s.FailNowf("invalid number of returned rows", "should be -1 or 2, got: %d", rows)
+	}
+	defer rdr.Release()
+
+	s.Truef(schema.Equal(utils.RemoveSchemaMetadata(rdr.Schema())), "expected: %s\n got: %s", schema, rdr.Schema())
+	s.Require().True(rdr.Next())
+	rec := rdr.Record()
+	s.EqualValues(2, rec.NumRows())
+	s.EqualValues(1, rec.NumCols())
+	col, ok := rec.Column(0).(*array.Int64)
+	s.True(ok)
+	s.Equal(int64(42), col.Value(0))
+	s.Equal(int64(42), col.Value(1))
+
+	s.Require().False(rdr.Next())
+	s.Require().NoError(rdr.Err())
+}
+
+func (s *StatementTests) TestSqlIngestErrors() {
+	if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeCreate) {
+		s.T().SkipNow()
+	}
+
+	s.Require().NoError(s.Quirks.DropTable(s.Cnxn, "bulk_ingest"))
+
 	stmt, err := s.Cnxn.NewStatement()
 	s.Require().NoError(err)
 	defer stmt.Close()
@@ -735,6 +1050,10 @@ func (s *StatementTests) TestSqlIngestErrors() {
 	})
 
 	s.Run("append to nonexistent table", func() {
+		if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeAppend) {
+			s.T().SkipNow()
+		}
+
 		s.Require().NoError(s.Quirks.DropTable(s.Cnxn, "bulk_ingest"))
 		schema := arrow.NewSchema([]arrow.Field{{
 			Name: "int64s", Type: arrow.PrimitiveTypes.Int64, Nullable: true}}, nil)
@@ -795,6 +1114,10 @@ func (s *StatementTests) TestSqlIngestErrors() {
 		batch = batchbldr.NewRecord()
 		defer batch.Release()
 
+		if !s.Quirks.SupportsBulkIngest(adbc.OptionValueIngestModeCreate) {
+			s.T().SkipNow()
+		}
+
 		s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestTargetTable, "bulk_ingest"))
 		s.Require().NoError(stmt.SetOption(adbc.OptionKeyIngestMode, adbc.OptionValueIngestModeAppend))
 		s.Require().NoError(stmt.Bind(s.ctx, batch))
