@@ -247,6 +247,18 @@ public class FlightSqlStatement implements AdbcStatement {
         new FlightInfoReader(allocator, client, clientCache, info.getEndpoints()));
   }
 
+  @Override
+  public Schema executeSchema() throws AdbcException {
+    if (bulkOperation != null) {
+      throw AdbcException.invalidState("[Flight SQL] Must executeUpdate() for bulk ingestion");
+    } else if (sqlQuery == null) {
+      throw AdbcException.invalidState("[Flight SQL] Must setSqlQuery() before execute");
+    }
+    return execute(
+        FlightSqlClient.PreparedStatement::getResultSetSchema,
+        (client) -> client.getExecuteSchema(sqlQuery).getSchema());
+  }
+
   @Override
   public UpdateResult executeUpdate() throws AdbcException {
     if (bulkOperation != null) {
