@@ -30,6 +30,7 @@ import java.util.Objects;
 import java.util.stream.LongStream;
 import org.apache.arrow.adapter.jdbc.JdbcFieldInfo;
 import org.apache.arrow.adapter.jdbc.JdbcParameterBinder;
+import org.apache.arrow.adapter.jdbc.JdbcToArrowConfig;
 import org.apache.arrow.adapter.jdbc.JdbcToArrowUtils;
 import org.apache.arrow.adbc.core.AdbcException;
 import org.apache.arrow.adbc.core.AdbcStatement;
@@ -263,6 +264,41 @@ public class JdbcStatement implements AdbcStatement {
     return new QueryResult(/*affectedRows=*/ -1, reader);
   }
 
+  @Override
+  public Schema executeSchema() throws AdbcException {
+    if (bulkOperation != null) {
+      throw AdbcException.invalidState("[JDBC] Call executeUpdate() for bulk operations");
+    } else if (sqlQuery == null) {
+      throw AdbcException.invalidState("[JDBC] Must setSqlQuery() first");
+    }
+    try {
+      invalidatePriorQuery();
+      final PreparedStatement preparedStatement;
+      final PreparedStatement ownedStatement;
+      if (statement instanceof PreparedStatement) {
+        preparedStatement = (PreparedStatement) statement;
+        if (bindRoot != null) {
+          JdbcParameterBinder.builder(preparedStatement, bindRoot).bindAll().build().next();
+        }
+        ownedStatement = null;
+      } else {
+        // new statement
+        preparedStatement = connection.prepareStatement(sqlQuery);
+        ownedStatement = preparedStatement;
+      }
+
+      final JdbcToArrowConfig config = JdbcArrowReader.makeJdbcConfig(allocator);
+      final Schema schema =
+          JdbcToArrowUtils.jdbcToArrowSchema(preparedStatement.getMetaData(), config);
+      if (ownedStatement != null) {
+        ownedStatement.close();
+      }
+      return schema;
+    } catch (SQLException e) {
+      throw JdbcDriverUtil.fromSqlException(e);
+    }
+  }
+
   @Override
   public Schema getParameterSchema() throws AdbcException {
     if (statement instanceof PreparedStatement) {
